﻿using System;
using System.Threading;
using Castle.DynamicProxy;
using Castle.MicroKernel.Registration;
using Castle.Windsor;

namespace CastleAbstraction.Proxies
{
    /// <summary>
    /// Proxy factory class
    /// </summary>
    public static class ProxyFactory
    {
        /// <summary>
        /// Initialize static members
        /// </summary>
        static ProxyFactory()
        {
            ProxyGenerator = new Lazy<ProxyGenerator>(
                () => new ProxyGenerator(), LazyThreadSafetyMode.PublicationOnly);
            WindsorContainer = new Lazy<WindsorContainer>(
                () => new WindsorContainer(), LazyThreadSafetyMode.PublicationOnly);
        }

        /// <summary>
        /// Proxy generator
        /// </summary>
        private static Lazy<ProxyGenerator> ProxyGenerator { get; set; }

        /// <summary>
        /// IoC container
        /// </summary>
        private static Lazy<WindsorContainer> WindsorContainer { get; set; }

        /// <summary>
        /// Use local mode is set to true, otherwise call remote service
        /// </summary>
        public static bool UseLocalExecutionMode { get; set; }

        /// <summary>
        /// Url that a service listens on
        /// </summary>
        public static string Url { get; set; }

        /// <summary>
        /// Create proxy of type T
        /// </summary>
        /// <typeparam name="T">Type of interface that a service implements</typeparam>
        /// <returns>Dynamic proxy</returns>
        public static T CreateProxy<T>()
            where T : class
        {
            if (UseLocalExecutionMode)
            {
                return WindsorContainer.Value.Resolve<T>();
            }
            return ProxyGenerator.Value.CreateInterfaceProxyWithoutTarget<T>
                (new ProxyInterceptor());
        }

 
        /// <summary>
        /// Register local services
        /// </summary>
        public static void RegisterTypes()
        {
            if (UseLocalExecutionMode)
            {
                WindsorContainer.Value.Register(Component.For<ITestService>().ImplementedBy<TestService>());
            }

        }
    }
}
